﻿using System;
using System.Security.Cryptography;
using System.Web.Script.Serialization;
using System.Web.Script.Services;
using System.Web.Services;

/// <summary>
/// Summary description for SigningService
/// </summary>
[WebService(Namespace = "http://tempuri.org/")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
// To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
[System.Web.Script.Services.ScriptService]
public class SigningService : System.Web.Services.WebService {

    public SigningService () {

        //Uncomment the following line if using designed components 
        //InitializeComponent(); 
    }

    /// <summary>
    /// Time-stamps and signs a URL
    /// </summary>
    /// <param name="url">The URL to sign</param>
    /// <returns>The signed URL</returns>
    [WebMethod]
    [ScriptMethod(UseHttpGet = true, ResponseFormat = ResponseFormat.Json)]
    public void SignUrl(string url)
    {
        // create a timestamp
        TimeSpan span = DateTime.UtcNow - new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);
        long ts = (long)Math.Floor(span.TotalSeconds);

        // time-stamp the url
        url += "&ts=" + ts;

        // generate the hash
        System.Text.ASCIIEncoding encoding = new System.Text.ASCIIEncoding();
        HMACMD5 hmac = new HMACMD5(encoding.GetBytes(Config.SecretKey));
        byte[] hash = hmac.ComputeHash(encoding.GetBytes(url));

        // convert hash to digital signature string
        string signature = BitConverter.ToString(hash).Replace("-", string.Empty).ToLower();

        // create the signed url
        string signedUrl = url + "&ds=" + signature;

        // Build a URL object
        Url outUrl = new Url()
        {
            url = signedUrl
        };

        // ASP.NET wants to return XML unless the request headers specifically ask for Json (stackoverflow.com/questions/2058454).
        // Here we return a JSON response ourselves
        Context.Response.Write(new JavaScriptSerializer().Serialize(outUrl));
    }
    
}
